/**
 * @file edp-webserver handler
 * @author wuqi03(wuqi03@baidu.com)
 */

var cssLoader = require('../index');
var fs = require('fs');
var path = require('path');
var crypto = require('crypto');

module.exports = function (options) {
    var inited = false;

    options = options || {};

    return function (context) {
        var docRoot  = context.conf.documentRoot;
        var pathname = context.request.pathname;
        var cache = path.resolve(docRoot, '.edpproj', 'edp-cache');


        if (!inited) {
            if (!fs.existsSync(path.resolve(docRoot, '.edpproj'))) {
                throw new Error('project is not an edp project, run `edp project init` first');
            }

            if (!fs.existsSync(cache)) {
                fs.mkdirSync(cache);
            }

            var ignore = path.resolve(docRoot, '.edpproj', '.gitignore');
            var content = fs.readFileSync(ignore, 'utf-8');

            if (!/^edp-cache$/m.test(content)) {
                content += '\nedp-cache';
                fs.writeFileSync(ignore, content, 'utf8');
            }

            inited = true;
        }

        var file = docRoot + pathname.replace( /\.js$/, '' );

        if (fs.existsSync(file)) {
            var code = fs.readFileSync(file, 'utf8');

            var hash = crypto.createHash('md5');
            hash.update(code);

            var md5 = hash.digest('hex');
            var cachePath = path.resolve(cache, 'cssloader_' + pathname.replace(/\/|\\|\./g, '_') + '.cache');

            // 判断缓存
            if (fs.existsSync(cachePath)) {
                var cacheContent = fs.readFileSync(cachePath, 'utf8');
                var backup = cacheContent.slice(0, 32);

                if (backup === md5) {
                    context.content = cacheContent.slice(32);
                    return;
                }
            }

            context.stop();

            cssLoader(
                file,
                options,
                function (fileText) {
                    context.content = fileText;
                    fs.writeFileSync(cachePath, md5 + fileText, 'utf8');

                    context.start();
                }
            );
        }
    };
};
